###########################################################
#
# General make-include
# Johan Bezem, JB Enterprises, (c) 2008.
#
# Prerequisites:
#  - GNUmake   3.79.1 or higher
#  - Bash      2.05a or higher
#    or
#  - CMD.EXE   Windows XP or higher
#
# Version 005.000
# Saved under "005Differentiate.zip"

# A few helpful variables go here
null:=
space:=$(strip $(null)) $(strip $(null))

# First, determine the name of the objects directory
# This name should be in such a way unique, that a pathname
# containing that string can only refer to an objects resp.
# intermediates' directory. This is of vital importance for
# the structure of the makefile.
export YT_OBJDIRNAME:=objects

# Record the directory from where we started, for later reference.
export YT_STARTDIR := $(CURDIR)

# Collect the components for the intermediates' directory
# in YT_DIFFDIR. Start out with 'Im' to avoid starting with a dash
# Use a simply expanded variable to enble self-references
YT_DIFFDIR := Im

# We must determine the operating system used.
# Initialize first:
YT_USE_WINDOWS:=0
YT_USE_LINUX  :=0
YT_USE_CMDEXE :=0
YT_USE_BASH   :=0

# If we are using Windows NT or later, the environment variable 
# OS will be set to 'Windows_NT'
ifneq (Windows_NT,$(OS))
# We'll assume Linux for the time being
  YT_USE_LINUX:=1
  YT_NAME_PLATFORM:=LINUX
  YT_USE_BASH:=1
  YT_NAME_SHELL:=BASH
else
  YT_USE_WINDOWS:=1
  YT_NAME_PLATFORM:=WINDOWS
  # We need to differentiate between Cygwin using bash, and 
  # Windows using CMD.EXE. For that, we look into the PATH variable, 
  # and search for semicolons; only in CMD.EXE, semicolons are 
  # allowed as path separators.
  ifeq (;,$(findstring ;,$(PATH)))
    YT_USE_CMDEXE:=1
    YT_NAME_SHELL:=CMDEXE
  else
    YT_USE_BASH:=1
    YT_NAME_SHELL:=BASH
  endif
endif

# And now export our findings for all recursions
export YT_USE_WINDOWS YT_USE_LINUX YT_USE_CMDEXE YT_USE_BASH
export YT_NAME_PLATFORM YT_NAME_SHELL

# Here we start with the host platform to define the architecure
include $(YT_PBASE)/make/platform_$(YT_NAME_PLATFORM).mk

# And then the shell's own specific definitions
include $(YT_PBASE)/make/shell_$(YT_NAME_SHELL).mk

# Now define the compiler (toolchain) platform
# See if the configuration requires a specific implementation
ifeq ($(origin YT_TC_SELECT),undefined)
  # If not, check for a default
  # CMD.EXE:
  ifeq (1,$(YT_USE_CMDEXE))
    # Take the newest supported VS
    YT_TC_SELECT := VS
  endif

  # Bash:
  ifeq (1,$(YT_USE_BASH))
    # Take the newest supported GCC
    YT_TC_SELECT := GCC
  endif

  # Test for a supported TC
  ifeq ($(origin YT_TC_SELECT),undefined)
    $(error No supported shell for default toolchain detection)
  endif
endif

# We now look for the concrete implementation
# We may find more than one, so sort the list and take the last one
# (with the highest version number)
# Using GNU make 3.81, the following line will work and be faster:
# YT_TC_MAKE_INCLUDE := $(lastword $(sort $(wildcard $(YT_PBASE)/make/tc_$(YT_TC_SELECT)*.mk)))
# For pre-3.81, the following equivalent is necessary
YT_TC_MAKE_INCLUDE := $(word $(words $(sort $(wildcard $(YT_PBASE)/make/tc_$(YT_TC_SELECT)*.mk))), $(sort $(wildcard $(YT_PBASE)/make/tc_$(YT_TC_SELECT)*.mk)))

ifeq (,$(strip $(YT_TC_MAKE_INCLUDE)))
  $(error Toolchain make include not found.)
endif

# And include the selected file
include $(YT_TC_MAKE_INCLUDE)

# All components for the intermediates' directory have been collected,
# so no make sure all make instances will inherit this value
export YT_DIFFDIR


